@@ -27,6 +27,12 @@ abstract class sspmod_core_Auth_UserPassOrgBase extends SimpleSAML_Auth_Source {
 	const AUTHID = 'sspmod_core_Auth_UserPassOrgBase.AuthId';
 
 
+	/**
+	 * The key of the OrgId field in the state, identifies which org was selected.
+	 */
+	const ORGID = 'sspmod_core_Auth_UserPassOrgBase.SelectedOrg';
+
+
 	/**
 	 * What way do we handle the organization as part of the username.
 	 * Three values:
@@ -194,6 +200,10 @@ abstract class sspmod_core_Auth_UserPassOrgBase extends SimpleSAML_Auth_Source {
 			return $e->getErrorCode();
 		}
 
+		// Add the selected Org to the state
+		$state[self::ORGID] = $organization;
+		$state['PersistentAuthData'][] = self::ORGID;
+
 		$state['Attributes'] = $attributes;
 		SimpleSAML_Auth_Source::completeAuth($state);
 	}
