@@ -11,7 +11,7 @@
  * @version $Id$
  */
  
-require_once('../../www/_include.php');
+require_once('../_include.php');
 
 $config = SimpleSAML_Configuration::getInstance();
 $session = SimpleSAML_Session::getInstance();
