@@ -2,20 +2,18 @@
 
 require_once('../../_include.php');
 
-$session = SimpleSAML_Session::getInstance();
-
 try {
 	$discoHandler = new SimpleSAML_XHTML_IdPDisco(array('shib13-idp-remote'), 'shib13');
 } catch (Exception $exception) {
 	/* An error here should be caused by invalid query parameters. */
-	SimpleSAML_Utilities::fatalError($session->getTrackID(), 'DISCOPARAMS', $exception);
+	throw new SimpleSAML_Error_Error('DISCOPARAMS', $exception);
 }
 
 try {
 	$discoHandler->handleRequest();
 } catch(Exception $exception) {
 	/* An error here should be caused by metadata. */
-	SimpleSAML_Utilities::fatalError($session->getTrackID(), 'METADATA', $exception);
+	throw new SimpleSAML_Error_Error('METADATA', $exception);
 }
 
 ?>
\ No newline at end of file
